@@ -1,7 +1,7 @@
 /* command line interpretation, image loading etc. for Gforth
 
 
-  Copyright (C) 1995,1996,1997,1998 Free Software Foundation, Inc.
+  Copyright (C) 1995,1996,1997,1998,2000 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -17,7 +17,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */
 
 #include "config.h"
@@ -50,7 +50,7 @@
 /* increment this whenever the primitives change in an incompatible way */
 
 #ifndef DEFAULTPATH
-#  define DEFAULTPATH "~+"
+#  define DEFAULTPATH "."
 #endif
 
 #ifdef MSDOS
@@ -130,9 +130,12 @@ void relocate(Cell *image, const char *bitstring, int size, Label symbols[])
   int i=0, j, k, steps=(size/sizeof(Cell))/RELINFOBITS;
   Cell token;
   char bits;
+  Cell max_symbols;
 
 /*  printf("relocating %x[%x]\n", image, size); */
-   
+  
+  for (max_symbols=DOESJUMP+1; symbols[max_symbols]!=0; max_symbols++)
+    ;
   size/=sizeof(Cell);
 
   for(k=0; k<=steps; k++) {
@@ -159,7 +162,10 @@ void relocate(Cell *image, const char *bitstring, int size, Label symbols[])
 	    default          :
 /*	      printf("Code field generation image[%x]:=CA(%x)\n",
 		     i, CF(image[i])); */
-	      image[i]=(Cell)CA(CF(token));
+	      if (CF(token)<max_symbols)
+		image[i]=(Cell)CA(CF(token));
+	      else
+		fprintf(stderr,"Primitive %d used in this image at $%lx is not implemented by this\n engine (%s); executing this code will crash.\n",CF(token),(long)&image[i],VERSION);
 	    }
 	else
 	  image[i]+=(Cell)image;
@@ -341,10 +347,11 @@ void alloc_stacks(ImageHeader * header)
 
 int go_forth(Address image, int stack, Cell *entries)
 {
-  ImageHeader *image_header = (ImageHeader *)image;
+  volatile ImageHeader *image_header = (ImageHeader *)image;
   Cell *sp0=(Cell*)(image_header->data_stack_base + dsize);
   Float *fp0=(Float *)(image_header->fp_stack_base + fsize);
   Cell *rp0=(Cell *)(image_header->return_stack_base + rsize);
+  volatile Cell *orig_rp0=rp0;
   Address lp0=image_header->locals_stack_base + lsize;
   Xt *ip0=(Xt *)(image_header->boot_entry);
 #ifdef SYSSIGNALS
@@ -371,7 +378,8 @@ int go_forth(Address image, int stack, Cell *entries)
     signal_data_stack[7]=throw_code;
 
 #ifdef GFORTH_DEBUGGING
-    if (rp <= rp0 && rp > (Cell *)(image_header->return_stack_base+5)) {
+    /* fprintf(stderr,"\nrp=%ld\n",(long)rp); */
+    if (rp <= orig_rp0 && rp > (Cell *)(image_header->return_stack_base+5)) {
       /* no rstack overflow or underflow */
       rp0 = rp;
       *--rp0 = (Cell)ip;
@@ -649,7 +657,7 @@ void gforth_args(int argc, char ** argv, char ** path, char ** imagename)
       /* no-init-file, no-rc? */
     };
     
-    c = getopt_long(argc, argv, "+i:m:d:r:f:l:p:vh", opts, &option_index);
+    c = getopt_long(argc, argv, "+i:m:d:r:f:l:p:vhoncsx", opts, &option_index);
     
     switch (c) {
     case EOF: return;
@@ -662,6 +670,11 @@ void gforth_args(int argc, char ** argv, char ** path, char ** imagename)
     case 'f': fsize = convsize(optarg,sizeof(Float)); break;
     case 'l': lsize = convsize(optarg,sizeof(Cell)); break;
     case 'p': *path = optarg; break;
+    case 'o': offset_image = 1; break;
+    case 'n': offset_image = 0; break;
+    case 'c': clear_dictionary = 1; break;
+    case 's': die_on_signal = 1; break;
+    case 'x': debug = 1; break;
     case 'v': fprintf(stderr, "gforth %s\n", VERSION); exit(0);
     case 'h': 
       fprintf(stderr, "Usage: %s [engine options] ['--'] [image arguments]\n\
